/*
 *	Noserc, based on University Corporation for Atmospheric Research gribdump.c.
 */
/* $Id: gribdump.c,v 1.20 1995/12/07 15:30:49 russ Exp $ */

/* Remember to update this constant for each change in the program */
//#define PROGVER "gribtocdl 1.1 - 18.12.2001"
#define PROGVER "gribtocdl 1.4 - 14.08.2002"

/* 
 * decodes GRIB products into ASCII
 */


#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <limits.h>		/* _POSIX_PATH_MAX */
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "ulog.h"
#include "mkdirs_open.h"

#include "centers.h"
#include "models.h"
#include "params.h"
#include "levels.h"
#include "timeunits.h"
#include "grib1.h"
#include "product_data.h"
#include "quasi.h"
#include "user_param.h"

#ifdef NO_ATEXIT
#include <atexit.h>
#endif

#define NR 250
#define NR_GDES 25

#include "test_noserc.h" 
int  testNoserc; 

/* Keep the original filename, if any - added 18.12.2001 by ab */
char* grib_file_name;

/* This struct is used to store the different levels in the struct gribdata */
typedef struct level{
int level_flg;       
char* clevelname;    
char* clevelsuffix;
/*int nparam;
int   param[NR]; */ 
int nflevel;          
float flevel[2][NR]; 
int   level[2][NR];
}level;

typedef struct parameter{
int param;
int nlevel;
int level_flg[NR];
}parameter;



typedef struct lgrib{
    char delim[4];	/* "GRIB", indicator block */ 
    int edition; /* 0 or 1, so far */
    int center;	/* NMC is 7 */
    int subcenter;	/* 0 if none */
}lgrib;


/* This struct is the datatype of the object m_gb where 
   the values from grib header information are 
   stored if not identical values already 
   exist in the object */
typedef struct gribdata{
/* this struct is based on the struct product_data in the file product_data.h */
int nlgrib;
lgrib* llgrib;
int nmodel;
int* model;  /* allocated by the originating center */
int ngrid;
int* grid;   /* Grid ID, may be center-specific */
int nparam_vers;
int* param_vers;/* parameter table version */
int nparam;          /*1*/
parameter* param;   /*1*//* Indicator of parameter */
int nlevel;          
level* gblevel;   
int nreftime;
char** creftime;
int ntunit;
int* tunit;
int ntr_flg;
int* tr_flg;/**/
int nhas_gds;
int* has_gds;
int nnpts;
int* npts;
int ngd;
gdes* gd; 
}gribdata;

typedef struct list_gribdata{
int ngb;
gribdata* gb;
}list_gribdata;

static gribdata m_gb;
static list_gribdata m_lgb;

#ifdef __STDC__
static int AddInt( int* arr, int narr, int val );
static int AddCharp( char** arr, int narr, char* val );
static void init_gribdata( gribdata* gb, int nr_gdes );
static void print_gribdata(int ngdes);
static void print_cdl(); 
static void add_grib(struct product_data* gp, int prec, int ngdes );
static int  do_add(FILE * fp, FILE * ep, int timeout, int prec, quas*);
static void cleanup(void);
static void signal_handler(int sig);
static void set_sigactions(void);
static void usage(char* av0);
static char* parameter_long_name( int i );
static char* level_long_name( int lev );
static void print_time_cdl();
static void print_levels(gribdata* l_gb);
static void print_parameters(gribdata* l_gb);
static int meta_level(int flag);
/*static void test_realloc( int reftime);*/
/*static void print_floats(float* ff, int cols, int rows, int pp);*/
static int is_ixg(int id);
/* static void print_grib(struct product_data* gp, int prec);*/
/*static void print_grib_line(struct product_data* gp);*/
/* static int do_dump(FILE * fp, FILE * ep, int timeout, int prec, quas*);*/
static product_data* grib_decode(prod*, quas*);
int main(int ac, char** av);
#endif

unsigned long num_wmo_messages; /* for statistics on exit */
unsigned long num_gribs_decoded;
unsigned long num_gribs_written;
unsigned long num_degribbed;



/*
 * Timeout in seconds.  If no input is received for this interval, process
 * closes output file and exits.
 */
#define DEFAULT_TIMEOUT  600

/*
 * Default precision used for printing floats.
 */
#define DEFAULT_PRECISION	7

/*
 * Called at exit.
 * This callback routine registered by atexit().
 */
static void
cleanup()
{
    uinfo("Exiting") ;
    uinfo("%lu WMO msgs, %lu GRIBs decoded, %lu written ",
	  num_wmo_messages, num_gribs_decoded, num_gribs_written);
    (void) closeulog();
}


/*
 * Called upon receipt of signals.
 * This callback routine registered in set_sigactions() .
 */
static void
signal_handler(sig)
     int sig ;
{
#ifdef SVR3SIGNALS
    /* 
     * Some systems reset handler to SIG_DFL upon entry to handler.
     * In that case, we reregister our handler.
     */
    (void) signal(sig, signal_handler) ;
#endif
    switch(sig) {
      case SIGHUP :
	udebug("SIGHUP") ;
	return ;
      case SIGINT :
	unotice("Interrupt") ;
	exit(0) ;
      case SIGTERM :
	udebug("SIGTERM") ;
	exit(0) ;
      case SIGUSR1 :
	udebug("SIGUSR1") ;
	return ;
      case SIGUSR2 :
	if (toggleulogpri(LOG_INFO))
	  unotice("Going verbose") ;
	else
	  unotice("Going silent") ;
	return ;
      case SIGPIPE :
	unotice("SIGPIPE") ;
	exit(0) ;
    }
    udebug("signal_handler: unhandled signal: %d", sig) ;
}


static void
set_sigactions()
{
#ifndef NO_POSIXSIGNALS
    struct sigaction sigact ;
    
    sigact.sa_handler = signal_handler;
    sigemptyset(&sigact.sa_mask) ;
    sigact.sa_flags = 0 ;
    
    (void) sigaction(SIGHUP, &sigact, NULL) ;
    (void) sigaction(SIGINT, &sigact, NULL) ;
    (void) sigaction(SIGTERM, &sigact, NULL) ;
    (void) sigaction(SIGUSR1, &sigact, NULL) ;
    (void) sigaction(SIGUSR2, &sigact, NULL) ;
    (void) sigaction(SIGPIPE, &sigact, NULL) ;
#else
    (void) signal(SIGHUP, signal_handler) ;
    (void) signal(SIGINT, signal_handler) ;
    (void) signal(SIGTERM, signal_handler) ;
    (void) signal(SIGUSR1, signal_handler) ;
    (void) signal(SIGUSR2, signal_handler) ;
    (void) signal(SIGPIPE, signal_handler) ;
#endif
}


static void
usage(av0)
     char *av0 ; /*  id string */
{
    (void)fprintf(stderr,
		  "Usage: %s [options] [files ...]\t\nOptions:\n", av0);
    (void)fprintf(stderr,
		  "\t-v           Verbose, report decoding steps\n") ;
    /*    (void)fprintf(stderr,
	  "\t-l logfile   Send log info to file (default uses stdout)\n") ; 
    (void)fprintf(stderr,
		  "\t-h           Output headers only, no data values\n") ;
    (void)fprintf(stderr,
    "\t-b           Brief output of headers data\n") ; */
    (void)fprintf(stderr,
		  "\t-g tabfile   Use grib-netcdf translation defined in this file\n") ;
    (void)fprintf(stderr,
		  "\t-t timeout   If no input, exit after \"timeout\" seconds (default %d)\n",
		  DEFAULT_TIMEOUT) ;
    /* (void)fprintf(stderr,
		  "\t-e errfile   Append bad GRIB products to this file\n") ;
    (void)fprintf(stderr,
		  "\t-p prec      Precision used to print floats (default %d)\n",
		  DEFAULT_PRECISION) ; */
    (void)fprintf(stderr,
		  "\t-q meth      Specs for expanding quasi-regular grids,\n\t\t\te.g. -q \"lin,dlat=2.5,dlon=5.0\"\n") ;
    exit(1);
}


int headers_only;
int brief_output;
int gdes_analyse;
int printcdl;



static int
is_ixg(id)			/* is an international exchange grid? */
     int id;
{
    switch (id) {
      case 21:
      case 22:
      case 23:
      case 24:
      case 25:
      case 26:
      case 37:
      case 38:
      case 39:
      case 40:
      case 41:
      case 42:
      case 43:
      case 44:
      case 50:
      case 61:
      case 62:
      case 63:
      case 64:
	return 1;
    }
    /* default: */
    return 0;
}

/*****************************************************/



static void print_gribdata( int ngdes ){
/* This function prints to screen from the objects m_lgb.gb[k] for all k, or the object m_gb */ 
 int i=0;
 int j=0;
 int k=0;
 int max=1;
 gribdata* l_gb;

 if(ngdes){ 
    max = m_lgb.ngb;
 }
 else 
    l_gb = &m_gb;

 for( k=0; k < max; k++ ){
   if(ngdes){ 
       l_gb = &m_lgb.gb[k];
   }
   printf("   %24s : %d\n","nmodel", l_gb->nmodel);
   for( i=0; i< l_gb->nmodel; i++ ){
     for( j=0; j<l_gb->nlgrib; j++ ){
	 printf("   %24s : %d %s\n","model", l_gb->model[i], modelname(l_gb->llgrib[j].center,l_gb->model[i]) );
     }
   }
   printf("   %24s : %d\n","ngrid", l_gb->ngrid);
   for( i=0; i< l_gb->ngrid; i++ )
     printf("   %24s : %d   %d \n","grid", l_gb->grid[i], is_ixg(l_gb->grid[i]));
   printf("   %24s : %d\n","nnpts", l_gb->nnpts);
   for( i=0; i< l_gb->nnpts; i++ )
     printf("   %24s : %d\n","npts", l_gb->npts[i]);
   printf("   %24s : %d\n","nparam", l_gb->nparam);
   for( i=0; i< l_gb->nparam; i++ )
     printf("   %24s : %d  %s  %s\n","param", l_gb->param[i].param, grib_pname(l_gb->param[i].param), grib_units(l_gb->param[i].param));

   printf("   %24s : %d\n","nlevel", l_gb->nlevel);
   for( i=0; i< l_gb->nlevel; i++ ){
      printf("   %24s : %d  %s : %s/%s \n","level_flg",l_gb->gblevel[i].level_flg,
	     "clevel",l_gb->gblevel[i].clevelname,l_gb->gblevel[i].clevelsuffix );
      printf("   %24s : %d\n","nflevel", l_gb->gblevel[i].nflevel);

      for( j=0; j<l_gb->gblevel[i].nflevel; j++){
	 int level[2];

	 if( l_gb->gblevel[i].flevel[0][j] >-1 )  
	    printf("   %24s : %f  ","flevel", l_gb->gblevel[i].flevel[0][j] );
	 if( l_gb->gblevel[i].flevel[1][j] > -1 )
	    printf("  %f", l_gb->gblevel[i].flevel[1][j] );

	 level[0] = l_gb->gblevel[i].level[0][j];
	 level[1] = l_gb->gblevel[i].level[1][j];
	 printf(" ** %4d %4d", level1(l_gb->gblevel[i].level_flg, level),
	       level2(l_gb->gblevel[i].level_flg, level) ); 
	 printf("\n");	
      }
   }

   printf("   %24s : %d\n","nreftime", l_gb->nreftime);
   for( i=0; i< l_gb->nreftime; i++ )
     printf("   %24s : %s\n","creftime", l_gb->creftime[i]);

   printf("   %24s : %d\n","ntunit", l_gb->ntunit);
   for( i=0; i< l_gb->ntunit; i++ )
     printf("   %24s : %s\n","tunitsname", tunitsname(l_gb->tunit[i]));
   printf("   %24s : %d\n","ntr_flg", l_gb->ntr_flg);
   for( i=0; i< l_gb->ntr_flg; i++ )
     printf("   %24s : %s\n","Time Range Indicator",  triname(l_gb->tr_flg[i]));
   printf("   %24s : %d\n","nhas_gds", l_gb->nhas_gds);
   for( i=0; i< l_gb->nhas_gds; i++ )
      printf("   %24s : %d\n","has_gds", l_gb->has_gds[i]);    
  printf("   %24s : %d\n","nedition==nlgrib", l_gb->nlgrib); 
  for( i=0; i< l_gb->nlgrib; i++ )
     printf("   %24s : %d\n","edition", l_gb->llgrib[i].edition);
   printf("   %24s : %d\n","nparam_vers", l_gb->nparam_vers);
   for( i=0; i< l_gb->nparam_vers; i++ )
     printf("   %24s : %d\n","param_vers", l_gb->param_vers[i]); 
   printf("   %24s : %d\n","ngd", l_gb->ngd);
   for( i=0; i< l_gb->ngd; i++ )
     print_gdes(&(l_gb->gd[i]));
 printf("****************************************** \n");
 printf("****************************************** \n");

 }/*end for k */
}


static void print_cdl(){
/* This function prints the cdl to standard output (the screen),
   the form of the cdl is based on an analysis of the grib code
   that stores data in the gribdata object m_gb */ 

 int i=0;
 int j=0;
 int k=0;
 /*int max=1;*/
 float ilon;
 float jlat;
 gribdata* l_gb=NULL;
 cdl_gdes_data* g_data=NULL;
 gdes* lgdes=NULL;
 int ngrids;
 
 
 /*int tab=8;
 char* ctab="        ";*/

 time_t t;
 char time_now[35];

/*
 if(ngdes){ 
    max = m_lgb.ngb;
    ngrids=m_lgb.ngb;
    l_gb = &m_lgb.gb[0]
 }
*/
 
 l_gb = &m_gb;
 ngrids=m_gb.ngd;

 /*
 for( k=0; k < max; k++ ){
   if(ngdes){ 
       l_gb = &m_lgb.gb[k];
   }
 */

 lgdes=&(l_gb->gd[0]);
 g_data= get_gdes_data(lgdes);

 printf("netcdf Noserc{ \n");
 printf("\n");
 printf("\n");
 printf("dimensions:\n");
 printf("        record = UNLIMITED ;    // (reference time, forecast time)\n");
 printf("        lat = %d ;\n",g_data->nj);
 printf("        lon = %d ;\n",g_data->ni);

 for( i=0; i<l_gb->nlevel; i++ ){
    int meta = meta_level(l_gb->gblevel[i].level_flg);

    if(meta==2){
	switch(l_gb->gblevel[i].level_flg){
	   case LEVEL_ISOBARIC:
	   {
	       printf("        level = %d ;\n", l_gb->gblevel[i].nflevel);
	    
	   }
	   break;
	   default:
	   {
	       char * suf=levelsuffix(l_gb->gblevel[i].level_flg);
	       printf("        %s = %d ;\n", suf,l_gb->gblevel[i].nflevel );
	   }
	}
    }
    if(meta==3){ 
       char * suf=levelsuffix(l_gb->gblevel[i].level_flg);
       printf("        %s = %d ;\n", suf,l_gb->gblevel[i].nflevel );
    }
    if(meta==-1){
       printf("ERROR");
    }
 }

 printf("        datetime_len = 21 ;\n");
 printf("        nmodels = %d ;\n", l_gb->nmodel);
 printf("        ngrids = %d ;\n", ngrids );
 printf("        nav = 1 ;\n");
 printf("        nav_len = 100 ;\n");
 printf("\n");
 printf("variables:\n");
 printf("\n");

 print_time_cdl();

 print_levels(l_gb);

 printf("\n");
 printf("	// The following lat and lon coordinate variables are redundant,\n");
 printf("	// since the navigation variables provide the necessary information.\n");
 printf("	// The extra information is included here for human readability.\n");
 printf("\n");
 printf("	float	lat(lat) ;\n");
 printf("		lat:long_name = \"latitude\" ;\n");
 printf("		lat:units = \"degrees_north\" ;\n");
 printf("\n");
 printf("	float	lon(lon) ;\n");
 printf("		lon:long_name = \"longitude\" ;\n");
 printf("		lon:units = \"degrees_east\" ;\n");
 printf("\n");
 printf("	long	model_id(nmodels) ;\n");
 printf("		model_id:long_name = \"generating process ID number\" ;\n");
 printf("\n");
 
 print_netcdf_gdes_nav(lgdes);

 print_parameters(l_gb);


 printf("\n");
 printf("// global attributes\n");
 /* updated traceability 18.12.2001 by ab */
 t = time(NULL);
 strftime(time_now,35,"%Y-%m-%d %T",localtime(&t));
 printf("                :history = \"%s - created by %s\" ; \n",
	time_now,PROGVER);
 printf("                :title = \"%s\" ;\n",grib_file_name);
 printf("                :Conventions = \"NUWG\" ;\n");
 printf("                :version = 0.0 ;\n");
 printf("\n");
 printf("data:\n");
 printf("\n"); 
 /* Updated 18.12.2001 by ab - print all model numbers */
 printf(" model_id = ");
 for ( i=0; i<l_gb->nmodel-1; i++ ) {
    printf("%d, ", l_gb->model[i]);
 }
 printf("%d ;\n",l_gb->model[l_gb->nmodel-1]);

 printf("\n");
 printf(" // Navigation\n");
 printf(" nav_model = \"GRIB1\" ;\n");
 printf(" grid_type_code = %d ;\n",l_gb->gd[0].type);
 printf(" grid_type = \"%s\" ;\n", gds_typename(l_gb->gd[0].type));
 printf(" grid_name = \" \" ;\n");
 if(l_gb->nlgrib>1) printf("ERROR, klarer ikke nlgrib>1");
 printf(" grid_center = %d ;\n",l_gb->llgrib[0].center);
 /* updated 18.12.2001 by ab - grid numbers are retrieved */
 printf(" grid_number = ");
 /* for ( i=1; i<ngrids; i++) printf("%d,",i);
    printf("%d ;\n",ngrids); */
 for ( i=0; i<ngrids-1; i++) printf("%d, ",l_gb->grid[i]);
    printf("%d ;\n",l_gb->grid[ngrids-1]); 
 printf(" i_dim = \"lon\" ;\n");
 printf(" j_dim = \"lat\" ;\n");
 
 print_netcdf_gdes_type(&(l_gb->gd[0]));

 printf(" %s = %d ;\n", "ResCompFlag", l_gb->gd[0].res_flags>0 ? 1: 0 );
 printf("\n");
 printf("\n");

 for( i=0; i<l_gb->nlevel; i++ ){
    int meta = meta_level(l_gb->gblevel[i].level_flg);

    if(meta==2){
	switch(l_gb->gblevel[i].level_flg){
	case LEVEL_ISOBARIC:
	{
	    printf(" level = ");    
	}
	break;
	default:
	{
	    char * suf=levelsuffix(l_gb->gblevel[i].level_flg);
	    printf(" %s = ", suf );   
	}
	break;
	}
	k=0;
	for( j=0; j<l_gb->gblevel[i].nflevel - 1; j++ ){
		printf("%.1f,", l_gb->gblevel[i].flevel[0][j]);
	        if(!(++k%8)){ printf("\n");printf("      ");}
	}
	    printf("%.1f ;\n", l_gb->gblevel[i].flevel[0][l_gb->gblevel[i].nflevel-1]);
    }
    if(meta==3){
	char* suf=levelsuffix(l_gb->gblevel[i].level_flg);
        printf(" %s_bot = ", suf );
	k=0;
	for( j=0; j<l_gb->gblevel[i].nflevel - 1; j++ ){    
	       printf("%.1f,", l_gb->gblevel[i].flevel[0][i]);
	       if(!(++k%8)){ printf("\n");printf("      ");}
	}
	printf("%.1f ;\n", l_gb->gblevel[i].flevel[0][l_gb->gblevel[i].nflevel-1]);
	
	k=0;
	printf(" %s_top = ", suf );
	for( j=0; j<l_gb->gblevel[i].nflevel - 1; j++ ){  
	       printf("%.1f,", l_gb->gblevel[i].flevel[1][i]);
	       if(!(++k%8)){ printf("\n");printf("      ");}
	}
        printf("%.1f ;\n", l_gb->gblevel[i].flevel[1][l_gb->gblevel[i].nflevel-1]);            
    }  
    if(meta==-1){
       printf("ERROR");
    }
  }/* end for( i=0; i<l_gb->nlevel; i++ ) */

    printf("\n");

    /* printf("i_dim = \"lon\""); */
    
	/* updated 18.12.2001 by ab 
           Adjust grid so that it is always South-North [-90:+90]
           and West-East [-180:180] or [0:360] */
        /* adjust numbers so that they are increasing */
	/* Assume longitude is allways west-east (sic) */
        while (g_data->lo2 < g_data->lo1) g_data->lo2 += 360;
        /* Just swap latitudes if wrong direction */
	if (g_data->la1 > g_data->la2) {
	  float tlat=g_data->la1;
	  g_data->la1=g_data->la2;
	  g_data->la2=tlat;
	}

    /* Write longitude grid points */

    ilon=g_data->lo1;
    
    printf("lon = ");
    k=0;
    while( ilon < g_data->lo2 ){
	printf("%6.2f,",ilon );  
        if(!(++k%8)){ printf("\n");printf("      ");}
	ilon+= g_data->di;
    }
    printf("%6.2f ;\n",ilon);

    printf("\n");

    /* Write latitude grid points */
    
    jlat=g_data->la1;
    printf("lat = ");
    k=0;
    while( jlat < g_data->la2 ){
	printf("%6.2f,",jlat );
        if(!(++k%8)){ printf("\n");printf("      ");}
	jlat+= g_data->dj;
    }
    printf("%6.2f ;\n",jlat);

    printf("\n");
    printf("}");

  return;
}


static void print_parameters( gribdata* l_gb ){
/* INPUT: gribdata* l_gb */
/* NB! se the paralell in the  print_levels function */
/* This function prints the parameter variables to the standard output */
/* CALLED BY: print_cdl()*/
 int i;
 int j;

  for( i=0; i< l_gb->nparam; i++ ){
     char* param= grib_pname(l_gb->param[i].param);
     for( j=0; j< l_gb->param[i].nlevel; j++ ){
	 char* datatype="float";
	 char * suf=levelsuffix(l_gb->param[i].level_flg[j]);
	 /*printf("start j< l_gb->param[i].nlevel\n"); */
	 
         if( meta_level(l_gb->param[i].level_flg[j]) ==1 )
	     printf("  %13s  %s_%s(record,lat,lon) ;\n", datatype, param,suf );
         else{
	    switch( l_gb->param[i].level_flg[j] ){
	      case LEVEL_ISOBARIC:
	      {
	        printf("  %13s  %s(record,level,lat,lon) ;\n", datatype, param );
	      }
	        break;
	      default:
	      {  
	        printf("  %13s  %s_%s(record,%s,lat,lon) ;\n", datatype, param,suf,suf);
	      }
                break;
	    }
	 }	 

	 switch( l_gb->param[i].level_flg[j] ){
	   case LEVEL_ISOBARIC:
	   {
	     printf("  %13s  %s:long_name = \"%s\" ;\n"," ", param, parameter_long_name( l_gb->param[i].param ) );
	     printf("  %13s  %s:units = \"%s\" ;\n"," ", param, grib_units(l_gb->param[i].param));
	     printf("  %13s  %s:navigation_var = \"nav\" ;\n"," ", param);
	   }
	   break;
	   default:
	   {
	     printf("  %13s  %s_%s:long_name = \"%s\" ;\n"," ", param, suf, parameter_long_name( l_gb->param[i].param ) );
	     printf("  %13s  %s_%s:units = \"%s\" ;\n"," ", param, suf, grib_units(l_gb->param[i].param));
	     printf("  %13s  %s_%s:navigation_var = \"nav\" ;\n"," ", param, suf );
	   }
	    break;
        } 
	printf("\n");
	/*printf("end j< l_gb->param[i].nlevel\n");*/
     }/* enf for j   */
  }
}


static void  print_levels(gribdata* l_gb){
/* INPUT: gribdata* l_gb */
/* This function prints the level variables to the standard output */
/* CALLED BY: print_cdl()*/
int i;
for( i=0; i<l_gb->nlevel; i++ ){
    int meta = meta_level(l_gb->gblevel[i].level_flg);

    if(meta==2){
	switch(l_gb->gblevel[i].level_flg){
	   case LEVEL_ISOBARIC:
	   {
	       
	       printf("       float   level(level) ;\n");
	       printf("               level:long_name = \"level\" ;\n");
	       printf("               level:units = \"hectopascals\" ;\n");
	       printf("\n");
	   }
	   break;
	   default:
	   {
	       char * suf=levelsuffix(l_gb->gblevel[i].level_flg);
	       char * unit=levelunits(l_gb->gblevel[i].level_flg);
	       printf("       float   %s(%s) ;\n", suf,suf);
               printf("               %s:long_name = ",suf);
               printf("\"%s\" ;\n",level_long_name( l_gb->gblevel[i].level_flg));
               printf("               %s:units = \"%s\" ;\n", suf, unit );
	       printf("\n");
	   }
	}
    }
    if(meta==3){ 
       char * suf=levelsuffix(l_gb->gblevel[i].level_flg);
       char * unit=levelunits(l_gb->gblevel[i].level_flg);
       printf(" %s:bot = \"%s_bot, %s_top\" ; // (\"%s_bot, %s_top\") uniquely\n", suf,suf ,suf,suf,suf);
       printf("                               // determines %s\n", suf );
       printf("\n");
       printf("	float	%s_bot(%s) ;\n", suf, suf );
       printf("		%s_bot:long_name = \"bottom level of ",suf);
       printf("%s \" ;\n",level_long_name( l_gb->gblevel[i].level_flg));
       printf("		%s_bot:units = \"%s\" ;\n", suf, unit );
       printf("\n");
       printf(" float	%s_top(%s) ;\n",suf,suf);
       printf("		%s_top:long_name = \"top level of ",suf); 
       printf("%s \" ;\n",level_long_name( l_gb->gblevel[i].level_flg));
       printf("		%s_top:units = \"%s\" ;\n",suf,suf);
       printf("\n");
    }
    if(meta==-1){
       printf("ERROR");
    }
}

}

/*
printf("	float	level(level) ;\n");
printf("		level:long_name = \"level\" ;\n");
printf("		level:units = \"hectopascals\" ;\n");
printf("\n");
printf("	:liso = \"liso_bot, liso_top\" ; // (liso_bot, liso_top) uniquely\n");
printf("				       // determines liso\n");
printf("\n");
printf("	float	liso_bot(liso) ;\n");
printf("		liso_bot:long_name = \"bottom level of layer between 2 isobaric levels\" ;\n");
printf("		liso_bot:units = \"hectopascals\" ;\n");
printf("\n");
printf("	float	liso_top(liso) ;\n");
printf("		liso_top:long_name = \"top level of layer between 2 isobaric levels\" ;\n");
printf("		liso_top:units = \"hectopascals\" ;\n");
printf("\n");
printf("	:lpdg = \"lpdg_bot, lpdg_top\" ; // (lpdg_bot, lpdg_top) uniquely\n");
printf("				       // determines lpdg\n");
printf("\n");
printf("	float	lpdg_bot(lpdg) ;\n");
printf("		lpdg_bot:long_name = \"bottom level of boundary layer between 2 levels at specified pressuredifferences from ground to levels\" ;\n");
printf("		lpdg_bot:units = "hectopascals" ;\n");
printf("\n");
printf("	float	lpdg_top(lpdg) ;\n");
printf("		lpdg_top:long_name = \"top level of boundary layer between 2 levels at specified pressure differences from ground to levels\" ;\n");
printf("		lpdg_top:units = \"hectopascals\" ;\n");
printf("\n");
printf("	float	fhg(fhg) ;		 // fixed height above ground\n");
printf("		fhg:long_name = \"fixed height above ground\" ;\n");
printf("		fhg:units = \"meters\" ;\n");
printf("\n");
printf("	float	sigma(sigma) ;		 // fixed height above ground\n");
printf("		sigma:long_name = \"sigma level\" ;\n");
printf("		sigma:units = \"\" ;	 // dimensionless\n");
*/


static int meta_level(flag)
/* This function is directly based on the function level1 in the file levels.c */
/* This function categorises the different levels in different meta levels. */
    int flag;			/* GRIB level flag */
{
    switch(flag){
    case LEVEL_SURFACE: 
    case LEVEL_CLOUD_BASE: 
    case LEVEL_CLOUD_TOP: 
    case LEVEL_ISOTHERM: 
    case LEVEL_ADIABAT: 
    case LEVEL_MAX_WIND: 
    case LEVEL_TROP: 
    case LEVEL_TOP:
    case LEVEL_SEABOT:
    case LEVEL_MEAN_SEA: 
    case LEVEL_ATM:
    case LEVEL_OCEAN:
    case LEVEL_HTFL:
    case LEVEL_LCBL:
    case LEVEL_LCTL:
    case LEVEL_LCY:
    case LEVEL_MCBL:
    case LEVEL_MCTL:
    case LEVEL_MCY:
    case LEVEL_HCBL:
    case LEVEL_HCTL:
    case LEVEL_HCY:
    case LEVEL_FL:
	return 1;
    case LEVEL_TMPL:
    case LEVEL_ISOBARIC: 
    case LEVEL_FH: 
    case LEVEL_FHG: 
    case LEVEL_SIGMA: 
    case LEVEL_HY:
    case LEVEL_Bls: 
    case LEVEL_ISEN:
    case LEVEL_PDG:
    case LEVEL_PV:
    case LEVEL_ETAL:
    case LEVEL_FHGH:
    case LEVEL_DBS: 
	return 2;	/* 2-octet level */
    case LEVEL_LISO: 
    case LEVEL_LFHM: 
    case LEVEL_LFHG: 
    case LEVEL_LS: 
    case LEVEL_LHY:
    case LEVEL_LBls: 
    case LEVEL_LISEN:
    case LEVEL_LPDG:
    case LEVEL_LETA: 
    case LEVEL_LISH: 
    case LEVEL_LSH: 
    case LEVEL_LISM: 
	return 3;		/* 1-octet level */
    }
    /* default */
    printf("ERROR");
    return -1;
}


static void print_time_cdl(){
/* This function prints the time variables to the standard output */
/* CALLED BY: print_cdl()*/
printf("	double	reftime(record) ;	// reference time of the model\n");
printf("		reftime:long_name = \"reference time\" ;\n");
printf("		reftime:units = \"hours since 1992-1-1\" ;\n");
printf("\n");
printf("	double	valtime(record) ;	// forecast time (\"valid\" time)\n");
printf("		valtime:long_name = \"valid time\" ;\n");
printf("		valtime:units = \"hours since 1992-1-1\" ;\n");
printf("\n");
printf("	:record = \"reftime, valtime\" ;	// \"dimension attribute\" -- means\n");
printf("					// (reftime, valtime) uniquely\n");
printf("					// determine record\n");
printf("\n");
printf("	char	datetime(record, datetime_len) ; // derived from reftime\n");
printf("		datetime:long_name = \"reference date and time\" ;\n");
printf("		// units YYYY-MM-DD hh:mm:ssZ  (ISO 8601)\n");
printf("\n");
printf("	float	valtime_offset(record) ; // derived as valtime-reftime\n");
printf("		valtime_offset:long_name = \"hours from reference time\" ;\n");
printf("		valtime_offset:units = \"hours\" ;\n");
printf("\n");
}






static char* level_long_name( int lev ){
/* this function is based upon the function levelsuffix in levels.c */
/* this function returns the longname to the different levels based on the levelflag */
    switch (lev){
    case LEVEL_SURFACE: return "surface of the Earth";
    case LEVEL_CLOUD_BASE: return "cloud base level";
    case LEVEL_CLOUD_TOP: return "cloud top level";
    case LEVEL_ISOTHERM: return "0 degree isotherm level";
    case LEVEL_ADIABAT: return "adiabatic condensation level";
    case LEVEL_MAX_WIND: return "maximium wind speed level";
    case LEVEL_TROP: return "at the tropopause";
    case LEVEL_TOP: return "nominal top of atmosphere";
    case LEVEL_SEABOT: return "sea bottom";
    case LEVEL_TMPL: return "temperature in 1/100 K";
    case LEVEL_ISOBARIC: return "isobaric level";
    case LEVEL_LISO: return "layer between two isobaric levels";
    case LEVEL_MEAN_SEA: return "mean sea level";
    case LEVEL_FH: return "fixed height level";
    case LEVEL_LFHM: return "layer between 2 height levels above MSL";
    case LEVEL_FHG: return "fixed height above ground";
    case LEVEL_LFHG: return "layer between 2 height levels above ground";
    case LEVEL_SIGMA: return "sigma level";
    case LEVEL_LS: return "layer between 2 sigma levels";
    case LEVEL_HY: return "Hybrid level";
    case LEVEL_LHY: return "Layer between 2 hybrid levels";
    case LEVEL_Bls: return "Depth below land surface";
    case LEVEL_LBls: return "Layer between 2 depths below land surface";
    case LEVEL_ISEN: return "Isentropic (theta) level ";
    case LEVEL_LISEN: return "Layer between 2 isentropic (theta) levels";
    case LEVEL_PDG: return "level at specified pressure difference from ground";
    case LEVEL_LPDG: return "layer between levels at specif. pressure diffs from ground";
    case LEVEL_PV: return "level of specified potential vorticity";
    case LEVEL_ETAL: return "ETA level";
    case LEVEL_LETA: return "layer between 2 ETA levels";
    case LEVEL_LISH: return "layer between 2 isobaric surfaces (high precision)";
    case LEVEL_FHGH: return "height level above ground (high precision)";
    case LEVEL_LSH: return "layer between 2 sigma levels (high precision)";
    case LEVEL_LISM: return "layer between 2 isobaric surfaces (mixed precision)";
    case LEVEL_DBS: return "depth below sea level";
    case LEVEL_ATM: return "entire atmosphere considered as a single layer";
    case LEVEL_OCEAN: return "entire ocean considered as a single layer";
    case LEVEL_HTFL: return "Highest tropospheric freezing level";
    case LEVEL_LCBL: return "Low cloud bottom level";
    case LEVEL_LCTL: return "Low cloud top level";
    case LEVEL_LCY: return "Low cloud layer";
    case LEVEL_MCBL: return "Middle cloud bottom level";
    case LEVEL_MCTL: return "Middle cloud top level";
    case LEVEL_MCY: return "Middle cloud layer";
    case LEVEL_HCBL: return "High cloud bottom level";
    case LEVEL_HCTL: return "High cloud top level";
    case LEVEL_HCY: return "Highcloud layer";
    case LEVEL_CCTH: return "convective cloud top height";
    case LEVEL_MEPTP: return "maximum equivalent potential temperature pressure";
    case LEVEL_ELH: return "equilibrium level height";
    case LEVEL_FL: return "FAA flight level";
    }
				/* default: */
    uerror("bad level flag: %d", lev);
    return "";

}


static char* parameter_long_name( int i ){
/* this function is based upon the definitions and parameters in params.h */
/* this function returns the longname to the different parameters based on the parameterflag */

  /* First check if anything was user-defined */
  char *p;
  p=user_plongname(i);
  if (p!=NULL) return p;

 switch (i) {    
    case PARM_RESERVED   : return "Reserved";
    case PARM_PRESSURE	 : return "Pressure, Pa";
    case PARM_PMSL	 : return "Pressure reduced to MSL, Pa";
    case PARM_PTND	 : return "Pressure tendency, Pa/s";
    case PARM_ICAHT	 : return "ICAO Standard Atmosphere Reference Height, m";
    case PARM_GPT	 : return "Geopotential, m2/s2";
    case PARM_GPT_HGT	 : return "Geopotential height, gpm";
    case PARM_GEOM_HGT	 : return "Geometric height, m";
    case PARM_HSTDV	    : return "Standard deviation of height, m";
    case PARM_TOZNE	    : return "Total Ozone, Dobsons";
    case PARM_TEMP	    : return "Temperature, deg. K";
    case PARM_VTEMP	    : return "Virtual temperature, deg. K";
    case PARM_POT_TEMP	    : return "Potential temperature, deg. K";
    case PARM_APOT_TEMP	    : return "Pseudo-adiabatic potential temperature, deg. K";
    case PARM_MAX_TEMP	    : return "Maximum temperature, deg. K";
    case PARM_MIN_TEMP	    : return "Minimum temperature, deg. K";
    case PARM_DP_TEMP	    : return "Dew point temperature, deg. K";
    case PARM_DP_DEP	    : return "Dew point depression (or deficit), deg. K";
    case PARM_LAPSE	    : return "Lapse rate, deg. K/m";

    case PARM_VIS	    : return "Visibility, m";

    case PARM_RAD1	    : return "Radar Spectra, direction and frequency";
    case PARM_RAD2	    : return "Radar Spectra, direction and radial num";
    case PARM_RAD3	    : return "Radar Spectra, radial num and radial num";
    case PARM_PLI	    : return "Parcel lifted index (to 500 hPa), K";

    case PARM_TANOM		: return "Temperature anomaly, deg. K";
    case PARM_PANOM		: return "Pressure anomaly, Pa";
    case PARM_ZANOM		: return "Geopotential height anomaly, gpm";
    case PARM_WAV1		: return "Wave Spectra, direction and frequency";
    case PARM_WAV2		: return "Wave Spectra, direction and radial num";
    case PARM_WAV3		: return "Wave Spectra, radial num and radial num";
    case PARM_WND_DIR		: return "Wind direction, deg. true";
    case PARM_WND_SPEED		: return "Wind speed, m/s";
    case PARM_U_WIND		: return "u-component of wind, m/s";
    case PARM_V_WIND		: return "v-component of wind, m/s";
    case PARM_STRM_FUNC		: return "Stream function, m2/s";
    case PARM_VPOT		: return "Velocity potential, m2/s";

    case PARM_MNTSF		: return "Montgomery stream function, m2/s2";

    case PARM_SIG_VEL		: return "Sigma coord. vertical velocity, /s";
    case PARM_VERT_VEL		: return "Pressure Vertical velocity, Pa/s";
    case PARM_GEOM_VEL		: return "Geometric Vertical velocity, m/s";
    case PARM_ABS_VOR		: return "Absolute vorticity, /s";
    case PARM_ABS_DIV		: return "Absolute divergence, /s";
    case PARM_REL_VOR		: return "Relative vorticity, /s";
    case PARM_REL_DIV		: return "Relative divergence, /s";
    case PARM_U_SHR		: return "Vertical u-component shear, /s";
    case PARM_V_SHR		: return "Vertical v-component shear, /s";
    case PARM_CRNT_DIR		: return "Direction of current, deg. true";
    case PARM_CRNT_SPD		: return "Speed of current, m/s";
    case PARM_U_CRNT		: return "u-component of current, m/s";
    case PARM_V_CRNT		: return "v-component of current, m/s";
    case PARM_SPEC_HUM 	: return "Specific humidity, kg/kg";
    case PARM_REL_HUM		: return "Relative humidity, %";
    case PARM_HUM_MIX		: return "Humidity mixing ratio, kg/kg";
    case PARM_PR_WATER		: return "Precipitable water, kg/m2";
    case PARM_VAP_PR		: return "Vapor pressure, Pa";
    case PARM_SAT_DEF		: return "Saturation deficit, Pa";
    case PARM_EVAP		: return "Evaporation, kg/m2";

    case PARM_C_ICE		: return "Cloud Ice, kg/m2";

    case PARM_PRECIP_RT		: return "Precipitation rate, kg/m2/s";
    case PARM_THND_PROB		: return "Thunderstorm probability, %";
    case PARM_PRECIP_TOT		: return "Total precipitation, kg/m2";
    case PARM_PRECIP_LS		: return "Large scale precipitation, kg/m2";
    case PARM_PRECIP_CN		: return "Convective precipitation, kg/m2";
    case PARM_SNOW_RT		: return "Snowfall rate water equivalent, kg/m2s";
    case PARM_SNOW_WAT		: return "Water equiv. of accum. snow depth, kg/m2";
    case PARM_SNOW		: return "Snow depth, m";
    case PARM_MIXED_DPTH		: return "Mixed layer depth, m";
    case PARM_TT_DEPTH		: return "Transient thermocline depth, m";
    case PARM_MT_DEPTH		: return "Main thermocline depth, m";
    case PARM_MTD_ANOM		: return "Main thermocline anomaly, m";
    case PARM_CLOUD		: return "Total cloud cover, %";
    case PARM_CLOUD_CN		: return "Convective cloud cover, %";
    case PARM_CLOUD_LOW		: return "Low cloud cover, %";
    case PARM_CLOUD_MED		: return "Medium cloud cover, %";
    case PARM_CLOUD_HI		: return "High cloud cover, %";
    case PARM_CLOUD_WAT		: return "Cloud water, kg/m2";
    case PARM_BLI		: return "Best lifted index (to 500 hPa), K";

    case PARM_SNO_C		: return "Convective snow, kg/m2";
    case PARM_SNO_L		: return "Large scale snow, kg/m2";

    case PARM_SEA_TEMP		: return "sea temperature in degrees K";
    case PARM_LAND_MASK		: return "Land-sea mask (1=land; 0=sea), 1/0";
    case PARM_SEA_MEAN		: return "Deviation of sea level from mean, m";
    case PARM_SRF_RN		: return "Surface roughness, m";
    case PARM_ALBEDO		: return "Albedo, %";
    case PARM_SOIL_TEMP		: return "Soil temperature, deg. K";
    case PARM_SOIL_MST 	: return "Soil moisture content, kg/m2";
    case PARM_VEG		: return "Vegetation, %";
    case PARM_SAL	 : return "Salinity, kg/kg";
    case PARM_DENS		: return "Density, kg/m3";

    case PARM_WATR		: return "Water runoff, kg/m2";

    case PARM_ICE_CONC		: return "Ice concentration (ice=l; no ice=O), 1/0";
    case PARM_ICE_THICK		: return "Ice thickness, m";
    case PARM_ICE_DIR		: return "Direction of ice drift, deg. true";
    case PARM_ICE_SPD		: return "Speed of ice drift, m/s";
    case PARM_ICE_U		: return "u-component of ice drift, m/s";
    case PARM_ICE_V		: return "v-component of ice drift, m/s";
    case PARM_ICE_GROWTH		: return "Ice growth, m";
    case PARM_ICE_DIV		: return "Ice divergence, /s";

    case PARM_SNO_M		: return "Snow melt, kg/m2";

    case PARM_WAVE_HGT		: return "Significant height of combined wind waves and swell, m";
    case PARM_SEA_DIR		: return "Direction of wind waves, deg. true";
    case PARM_SEA_HGT		: return "Significant height of wind waves, m";
    case PARM_SEA_PER		: return "Mean period of wind waves, s";
    case PARM_SWELL_DIR		: return "Direction of swell waves, deg. true";
    case PARM_SWELL_HGT		: return "Significant height of swell waves, m";
    case PARM_SWELL_PER		: return "Mean period of swell waves, s";
    case PARM_WAVE_DIR		: return "Primary wave direction, deg. true";
    case PARM_WAVE_PER		: return "Primary wave mean period, s";
    case PARM_WAVE2_DIR		: return "Secondary wave direction, deg. true";
    case PARM_WAVE2_PER		: return "Secondary wave mean period, s";
    case PARM_RDN_SWSRF		: return "Net shortwave radiation (surface), W/m2";
    case PARM_RDN_LWSRF		: return "Net longwave radiation (surface), W/m2";
    case PARM_RDN_SWTOP		: return "Net shortwave radiation (top of atmos.), W/m2";
    case PARM_RDN_LWTOP		: return "Net longwave radiation (top of atmos.), W/m2";
    case PARM_RDN_LW		: return "Long wave radiation, W/m2";
    case PARM_RDN_SW		: return "Short wave radiation, W/m2";
    case PARM_RDN_GLBL		: return "Global radiation, W/m2";
    case PARM_BRTMP		: return "Brightness temperature, K";
    case PARM_LWRAD		: return "Long-wave radiation, W/srm2";
    case PARM_SWRAD		: return "Short-wave radiation, W/srm2";

    case PARM_LAT_HT		: return "Latent heat flux, W/m2";
    case PARM_SEN_HT		: return "Sensible heat flux, W/m2";
    case PARM_BL_DISS		: return "Boundary layer dissipation, W/m2";

    case PARM_U_FLX		: return "Momentum flux, u component, N/m2";
    case PARM_V_FLX		: return "Momentum flux, v component, N/m2";
    case PARM_WMIXE		: return "Wind mixing energy, J";

    case PARM_IMAGE		: return "Image data";
    case PARM_MSLSA		: return "Mean sea level pressure (std. atms. reduction), Pa";
    case PARM_PM			: return "Mean sea level pressure (MAPS system reduction), Pa";
    case PARM_MSLET		: return "Mean sea level pressure (ETA model reduction), Pa";
    case PARM_LIFT_INDX		: return "Surface lifted index, Deg. K";
    case PARM_LIFT_INDX4		: return "Best (4 layer) lifted index, Deg. K";
    case PARM_K_INDX		: return "K index, Deg. K";
    case PARM_SW_INDX		: return "Sweat index, Deg. K";
    case PARM_HM_DIV		: return "Horizontal moisture divergence, kg/kg/s";
    case PARM_VERT_SSHR		: return "Vertical speed shear, /s";
    case PARM_TSLSA		: return "3-hr pressure tendency, Std. Atmos. Reduction, Pa/s";
    case PARM_BVF_2		: return "Brunt-Vaisala frequency (squared), 1/s2";
    case PARM_PV_MW		: return "Potential vorticity (density weighted), 1/s/m";
    case PARM_CRAIN		: return "Categorical rain  (yes=1; no=0), non-dim";
    case PARM_CFRZRN		: return "Categorical freezing rain  (yes=1; no=0), non-dim";
    case PARM_CICEPL		: return "Categorical ice pellets  (yes=1; no=0), non-dim";
    case PARM_CSNOW		: return "Categorical snow  (yes=1; no=0), non-dim";
    case PARM_COVMZ		: return "Covariance between meridional and zonal components of wind, m2/s2";
    case PARM_COVTZ		: return "Covariance between temperature and zonal component of wind, K*m/s";
    case PARM_COVTM		: return "Covariance between temperature and meridional component of wind, K*m/s";
    case PARM_CLWMR		: return "Cloud water, kg/kg";
    case PARM_O3MR		: return "Ozone mixing ratio, kg/kg";

    case PARM_GFLUX		: return "Ground Heat Flux, W/m2";
    case PARM_CIN		: return "Convective inhibition, J/kg";
    case PARM_CAPE		: return "Convective Available Potential Energy, J/kg";
    case PARM_TKE		: return "Turbulent Kinetic Energy, J/kg";
    case PARM_CONDP		: return "Condensation pressure of parcel lifted from indicated surface, Pa";
    case PARM_CSUSF		: return "Clear Sky Upward Solar Flux, W/m2";
    case PARM_CSDSF		: return "Clear Sky Downward Solar Flux, W/m2";
    case PARM_CSULF		: return "Clear Sky upward long wave flux, W/m2";
    case PARM_CSDLF		: return "Clear Sky downward long wave flux, W/m2";
    case PARM_CFNSF		: return "Cloud forcing net solar flux, W/m2";
    case PARM_CFNLF		: return "Cloud forcing net long wave flux, W/m2";
    case PARM_VBDSF		: return "Visible beam downward solar flux, W/m2";
    case PARM_VDDSF		: return "Visible diffuse downward solar flux, W/m2";
    case PARM_NBDSF		: return "Near IR beam downward solar flux, W/m2";
    case PARM_NDDSF		: return "Near IR diffuse downward solar flux, W/m2";
    case PARM_M_FLX		: return "Momentum flux, N/m2";
    case PARM_LMH		: return "Mass point model surface, non-dim";
    case PARM_LMV		: return "Velocity point model surface, non-dim";
    case PARM_MLYNO		: return "Model layer number (from bottom up), non-dim";
    case PARM_NLAT    	: return "latitude (-90 to +90), deg";
    case PARM_ELON		: return "east longitude (0-360), deg";
    case PARM_LPS_X		: return "x-gradient of log pressure, 1/m";
    case PARM_LPS_Y		: return "y-gradient of log pressure, 1/m";
    case PARM_HGT_X		: return "x-gradient of height, m/m";
    case PARM_HGT_Y		: return "y-gradient of height, m/m";

    case PARM_VPTMP		: return "Virtual potential temperature, K";
    case PARM_HLCY		: return "Storm relative helicity, m2/s2";
    case PARM_PROB		: return "Probability from ensemble, percent";
    case PARM_PROBN : return "Probability from ensemble normalized with respect to climate expectancy, percent";
    case PARM_POP		: return "Probability of precipitation, percent";
    case PARM_CPOFP		: return "Probability of frozen precipitation, percent";
    case PARM_CPOZP		: return "Probability of freezing precipitation, percent";
    case PARM_USTM		: return "u-component of storm motion, m/s";
    case PARM_VSTM		: return "v-component of storm motion, m/s";
    case PARM_NOICE_WAT		: return "Ice-free water surface, %";

    case PARM_DSWRF		: return "downward short wave rad. flux, W/m2";
    case PARM_DLWRF		: return "downward long wave rad. flux, W/m2";
    case PARM_UVPI		: return "Ultra violet potential index, W/m2";
    case PARM_MSTR_AVL		: return "Moisture availability, %";
    case PARM_XCHG_COF		: return "Exchange coefficient, (kg/m3)(m/s)";
    case PARM_NMIX_LYRS		: return "No. of mixed layers next to surface, integer";

    case PARM_USWRF		: return "upward short wave rad. flux, W/m2";
    case PARM_ULWRF		: return "upward long wave rad. flux, W/m2";

    case PARM_CLOUD_NCN		: return "Amount of non-convective cloud, %";

    case PARM_CPRAT		: return "Convective Precipitation rate, kg/m2/s";
    case PARM_TTDIA		: return "Temperature tendency by all physics, K/s";

    case PARM_RDN_TTND		: return "Temperature tendency by all radiation, Deg.K/s";

    case PARM_TTPHY		: return "Temperature tendency by non-radiation physics, K/s";
    case PARM_PREIX		: return "precip.index(0.0-1.00)(see note), fraction";
    case PARM_TSD1D		: return "Std. dev. of IR T over 1x1 deg area, K";

    case PARM_LN_PRES		: return "Natural log of surface pressure, ln(kPa)";
    case PARM_HPBL		: return "Planetary boundary layer height, m";
    case PARM_GPT_HGT5		: return "5-wave geopotential height, gpm";

    case PARM_C_WAT		: return "Plant canopy surface water, kg/m2";
    case PARM_BMIXL		: return "Blackadar's mixing length scale, m";
    case PARM_AMIXL		: return "Asymptotic mixing length scale, m";
    case PARM_PEVAP		: return "Potential evaporation, kg/m2";
    case PARM_SNOHF		: return "Snow phase-change heat flux, W/m2";
    case PARM_MFLUX		: return "Convective cloud mas flux, Pa/s";
    case PARM_DTRF		: return "Downward total radiation flux, W/m2";
    case PARM_UTRF		: return "Upward total radiation flux, W/m2";
    case PARM_BGRUN	 	: return "Baseflow-groundwater runoff, kg/m2";
    case PARM_SSRUN		: return "Storm surface runoff, kg/m2";
    case PARM_03TOT		: return "Total ozone, Kg/m2";
    case PARM_SNO_CVR		: return "Snow cover, percent";
    case PARM_SNO_T		: return "Snow temperature, K";
    case PARM_LRGHR		: return "Large scale condensat. heat rate, K/s";
    case PARM_CNVHR		: return "Deep convective heating rate, K/s";
    case PARM_CNVMR		: return "Deep convective moistening rate, kg/kg/s";
    case PARM_SHAHR		: return "Shallow convective heating rate, K/s";
    case PARM_SHAMR		: return "Shallow convective moistening rate, kg/kg/s";
    case PARM_VDFHR		: return "Vertical diffusion heating rate, K/s";
    case PARM_VDFUA		: return "Vertical diffusion zonal acceleration, m/s2";
    case PARM_VDFVA		: return "Vertical diffusion meridional accel, m/s2";
    case PARM_VDFMR		: return "Vertical diffusion moistening rate, kg/kg/s";
    case PARM_SWHR		: return "Solar radiative heating rate, K/s";
    case PARM_LWHR		: return "long wave radiative heating rate, K/s";
    case PARM_CD		: return "Drag coefficient, non-dim";
    case PARM_FRICV		: return "Friction velocity, m/s";
    case PARM_RI		: return "Richardson number, non-dim.";

    case PARM_MISSING		: return "Missing";
 }
 return "Parameter not found";
}


static void init_gribdata( gribdata* gb, int nr_gdes ){
/* This function innitialises a struct of type gribdata */
   /* printf("sizeof(char*) = %d", sizeof(char*)); */
   gb->nlgrib=0;
   gb->llgrib=(lgrib*)malloc(sizeof(lgrib)*nr_gdes);
   gb->nmodel=0;
   gb->model=(int*)malloc(sizeof(int)*NR);
   gb->ngrid=0;
   gb->grid=(int*)malloc(sizeof(int)*NR);
   gb->nparam_vers=0;
   gb->param_vers=(int*)malloc(sizeof(int)*NR);
   gb->nparam=0;
   gb->param=(parameter*)malloc(sizeof(parameter)*NR);
   gb->nlevel=0;
   gb->gblevel=(level*)malloc(sizeof(level)*NR);
   gb->nreftime=0;
   gb->creftime=(char**)malloc(sizeof(char*)*NR);
   gb->ntunit=0;
   gb->tunit=(int*)malloc(sizeof(int)*NR);
   gb->ntr_flg=0;
   gb->tr_flg=(int*)malloc(sizeof(int)*NR);
   gb->nhas_gds=0;
   gb->has_gds=(int*)malloc(sizeof(int)*NR);
   gb->nnpts=0;
   gb->npts=(int*)malloc(sizeof(int)*NR);
   gb->ngd=0;
   gb->gd=(gdes*)malloc(sizeof(gdes)*nr_gdes);
   return;
}
/*
int ngd;
gdes* gd; 
}gribdata;

typedef struct list_gribdata{
int ngb;
gribdata* gb;
}list_gribdata;

static gribdata m_gb;
static list_gribdata m_lgb;
*/


static void
add_grib(gp, prec, ngdes )
     struct product_data *gp;
     int prec;			/* precision to use for printing float vals */
     int ngdes;                  /* is one if one orders the data by gdes */ 
{
/* This function is originally based on the function print_grib,
   but instead of printing the values to the terminal screen they are 
   stored in the object struct static gribdata m_gb or m_lgb.gb if not identical values already 
   are stored in the struct. The INPUT parameter ngdes control wether the data is ordered
   in a structure after gdes type(m_lgb.gb) or not( m_gb). 
*/
    char* ps;
    int b=0;
    int i=0;
    int j=0;
    int index=0;
    gribdata* l_gb=NULL;
    

    float lflevel[2];
    lflevel[0]= -1;
    lflevel[1]= -1;

    if(ngdes){
	int index_lgb=0;
	b=0;
	
	for( i=0; i< m_lgb.ngb; i++ ){
	    /*m_lgb.gb[i].gd[0].keep=0; */
	    if(gdes_cmp( gp->gd, m_lgb.gb[i].gd )){
		b=1; index_lgb=i; break;
	    }
	   /* m_lgb.gb[i].gd[0].keep=1;*/
	}

	if(!b){
	    if( !(m_lgb.ngb%NR_GDES) && m_lgb.ngb !=0 ){/* reallocate more memory*/	        
		if(!(m_lgb.gb=(gribdata*)realloc(m_lgb.gb,
			    (sizeof(gribdata)*( m_lgb.ngb + NR_GDES)))))
		    uerror("Error: can't alloc for parameter array: %d", m_lgb.ngb );
		/*
		  for( i=m_lgb.ngb; i<(m_lgb.ngb + NR_GDES); i++ )
		      m_lgb.gb[i]=NULL;*/
	    }/* end reallocate more memory*/

	    init_gribdata( &m_lgb.gb[m_lgb.ngb],1 );
	    index_lgb = m_lgb.ngb;
	    m_lgb.gb[m_lgb.ngb].ngd=1;
	    gp->gd->keep=1;/* This protects the memory against free_gdes called by the 
			      function free_product_data*/
	    m_lgb.gb[m_lgb.ngb].gd[0] = *(gp->gd); 
	    m_lgb.ngb++;	
	}

	l_gb = &m_lgb.gb[index_lgb];

    }
    else{
	 b=0;

	 for( i=0; i< m_gb.ngd; i++ ){
	     /*m_gb.gd[i].keep=0;*/
	     if(gdes_cmp( gp->gd, &(m_gb.gd[i]) )){
		 b=1;break;
	     }
	     /*m_gb.gd[i].keep=1;*/
	 }

	 if(!b){
	    if( !(m_gb.ngd%NR_GDES) && m_gb.ngd !=0 ){/* reallocate more memory*/	        
		if(!(m_gb.gd=(gdes*)realloc(m_gb.gd,
			    (sizeof(gribdata)*( m_gb.ngd + NR_GDES)))))
		    uerror("Error: can't alloc for parameter array: %d", m_gb.ngd );
		/*
		  for( i=m_gb.ngd; i<(m_gb.ngd + NR_GDES); i++ )
		      m_gb.gd[i]=NULL;*/
	    }/* end reallocate more memory*/

	     
	     gp->gd->keep=1;/* This protects the memory against free_gdes called by the 
			       function free_product_data*/
	     /*copy_gdes( gp->gd, &(m_gb.gd[i]) );*/
	     m_gb.gd[m_gb.ngd] = *(gp->gd);
	     m_gb.ngd++;
	 }
	 l_gb = &m_gb;
    }
   

/*
    printf("   %24s : %s\n","    Header", gp->header);
    printf("   %24s : %d (%s)\n","Originating Center", gp->center,
	   centername(gp->center));
    if(gp->subcenter != 0)
    printf("   %33s : %d (%s)\n","Subcenter", gp->subcenter,
	   subcentername(gp->center, gp->subcenter));
*/

     if( !(l_gb->nlgrib%NR_GDES) ){	        
	if(!(l_gb->llgrib=(lgrib*)realloc(l_gb->llgrib,
			    (sizeof(char*)*(l_gb->nlgrib + NR_GDES)))))
        uerror("Error: can't alloc for creftime array: %d", l_gb->nlgrib);
    }
    
    b=0;
    for( i=0; i< l_gb->nreftime; i++ ){
	if( (!strcmp(l_gb->llgrib[i].delim,gp->delim)) && l_gb->llgrib[i].edition==gp->edition &&
	    l_gb->llgrib[i].center==gp->center && l_gb->llgrib[i].subcenter==gp->subcenter ){
	    b=1;break;
	}
    }
    if(!b){
	strcpy(l_gb->llgrib[l_gb->nlgrib].delim,gp->delim);
	l_gb->llgrib[l_gb->nlgrib].edition=gp->edition;
	l_gb->llgrib[l_gb->nlgrib].center=gp->center;
	l_gb->llgrib[l_gb->nlgrib].subcenter=gp->subcenter;
	l_gb->nlgrib++;
    }


    /* printf("l_gb->nreftime= %d\n", l_gb->nreftime); */
 
    l_gb->nmodel=AddInt( l_gb->model, l_gb->nmodel, gp->model);
/*
    printf("   %24s : %d\n","Grid", gp->grid);
*/
    l_gb->ngrid=AddInt( l_gb->grid, l_gb->ngrid, gp->grid);

/*
    printf("             points in grid : %d\n", gp->npts);
*/

    l_gb->nnpts=AddInt( l_gb->npts, l_gb->nnpts, gp->npts);
 
/*
    printf("   %24s : %d (%s)\n","Parameter", gp->param,
	   grib_pname(gp->param));
    printf("   %24s : %s\n", "Units", grib_units(gp->param));
*/
    if( !(l_gb->nparam%NR) && l_gb->nparam !=0 ){	        
	if(!(l_gb->param=(parameter*)realloc(l_gb->param,
			    (sizeof(parameter)*(l_gb->nparam + NR)))))
        uerror("Error: can't alloc for parameter array: %d", l_gb->nparam);
	/*
	for( i=l_gb->nparam; i<(l_gb->nparan + NR); i++ )
		l_gb->param[i]=NULL;
	*/
    }

    l_gb->param[l_gb->nparam].param=gp->param;
    
    b=0;
    index=0;
    for( i=0; i< l_gb->nparam; i++ ){
	if( l_gb->param[l_gb->nparam].param == l_gb->param[i].param ){
	   b=1;
	   index=i;
	   break;
	}
    }

    if(b){
	int b_parlev=0; /* here this variable means boolean control for the level */ 
        for( i=0; i< l_gb->param[index].nlevel; i++ ){
	    if( gp->level_flg == l_gb->param[index].level_flg[i] ){
		b_parlev=1;break;
	    }
	}
	if(!b_parlev){
	    l_gb->param[index].level_flg[l_gb->param[index].nlevel]=gp->level_flg;
	    l_gb->param[index].nlevel++;
	}
	    
    }
    else{ /*if(!b)*/
	
	for( j=0; j<NR; j++ ){  
	    l_gb->param[l_gb->nparam].level_flg[j]=-1;/* initialise values */
	}
	l_gb->param[l_gb->nparam].nlevel = 1; /* the new parameter has at least one level */
        l_gb->param[l_gb->nparam].level_flg[0] = gp->level_flg;
	l_gb->nparam++;
    } 
   
/*
    printf("   %24s : %s\n", "Level Type", levelname(gp->level_flg));
*/

   if( !(l_gb->nlevel%NR) && l_gb->nlevel!=0 ){	        
	if(!(l_gb->gblevel=(level*)realloc(l_gb->gblevel,
			    (sizeof(level)*(l_gb->nlevel + NR)))))
        uerror("Error: can't alloc for level array: %d", l_gb->nlevel);
	/*
	for( i=l_gb->nlevel; i<(l_gb->nlevel + NR); i++ )
		l_gb->gblevel[i]=NULL;
	*/
    } 


    l_gb->gblevel[l_gb->nlevel].level_flg = gp->level_flg;            
    
    b=0;
    index=0;/* index is the index of position in the array 'struct level gblevel[NR]' */
    for( i=0; i< l_gb->nlevel; i++ ){
	if( l_gb->gblevel[l_gb->nlevel].level_flg == l_gb->gblevel[i].level_flg ){
	   b=1;
	   index=i;
	   break;
	}
    }

    if(!b){
        l_gb->gblevel[l_gb->nlevel].level_flg=gp->level_flg;
        l_gb->gblevel[l_gb->nlevel].clevelname = levelname(gp->level_flg);
	l_gb->gblevel[l_gb->nlevel].clevelsuffix = levelsuffix(gp->level_flg);
        index=l_gb->nlevel;
        l_gb->gblevel[index].nflevel=0; /* initialise values */
	for( j=0; j<NR; j++ ){  /* initialise values */
	    l_gb->gblevel[index].flevel[0][j]=-1;
	    l_gb->gblevel[index].flevel[1][j]=-1;
	    l_gb->gblevel[index].level[0][j]=-1;
	    l_gb->gblevel[index].level[1][j]=-1;
	}
	    
        l_gb->nlevel++;
    }    

    switch(meta_level(gp->level_flg)) {
    case 1:
	break;
    case 2:      
	lflevel[0] = level1(gp->level_flg,gp->level);
	/* lflevel[1] = -1; */
    case 3:
	lflevel[0] = level1( gp->level_flg, gp->level );
        lflevel[1] = level2( gp->level_flg, gp->level );
	break;
    }
      
    b=0;
    for( i=0; i< l_gb->gblevel[index].nflevel; i++ ){
	if( ( lflevel[0] == l_gb->gblevel[index].flevel[0][i] ) && 
            ( lflevel[1] == l_gb->gblevel[index].flevel[1][i] ) &&
	    ( gp->level[0] == l_gb->gblevel[index].level[0][i] ) && 
            ( gp->level[1] == l_gb->gblevel[index].level[1][i] )
	    ){
	    b=1;break;
	}
    }

    if(!b){ 
	l_gb->gblevel[index].flevel[0][l_gb->gblevel[index].nflevel] = lflevel[0];
        l_gb->gblevel[index].flevel[1][l_gb->gblevel[index].nflevel] = lflevel[1];
        l_gb->gblevel[index].level[0][ l_gb->gblevel[index].nflevel] = gp->level[0];
	l_gb->gblevel[index].level[1][ l_gb->gblevel[index].nflevel] = gp->level[1];
   	l_gb->gblevel[index].nflevel++;
    }

    ps=(char*)malloc(80);
    sprintf( ps," %04d/%02d/%02d:%02d:%02d", 
				/* century 21 doesn't start until 2001 */
	   gp->year+(gp->century - (gp->year==0 ? 0 : 1))*100,
	   gp->month, gp->day, gp->hour, 
	   gp->minute);


    if( !(l_gb->nreftime%NR) ){	        
	if(!(l_gb->creftime=(char**)realloc(l_gb->creftime,
			    (sizeof(char*)*(l_gb->nreftime + NR)))))
        uerror("Error: can't alloc for creftime array: %d", l_gb->nreftime);

	for( i=l_gb->nreftime; i<(l_gb->nreftime + NR); i++ )
		l_gb->creftime[i]=NULL;
    }


    l_gb->creftime[l_gb->nreftime]=ps;
    
    b=0;
    for( i=0; i< l_gb->nreftime; i++ ){
	if(!strcmp( l_gb->creftime[l_gb->nreftime], l_gb->creftime[i] )){
	   b=1;
	}
    }
    if(!b){
	l_gb->nreftime++;
    }

    /* printf("l_gb->nreftime= %d\n", l_gb->nreftime); */
    
/*    
    printf("   %24s : %s\n", "Time Unit", tunitsname(gp->tunit));
*/
    l_gb->ntr_flg= AddInt( l_gb->tr_flg, l_gb->ntr_flg, gp->tr_flg );
/*
    printf("   %24s : %s\n", "Time Range Indicator", triname(gp->tr_flg));
*/
    l_gb->ntr_flg= AddInt( l_gb->tr_flg, l_gb->ntr_flg, gp->tr_flg );


/*
    switch (gp->tr_flg) {
      case TRI_P1:
      case TRI_IAP:
	printf("   %24s : %d\n", "Time 1 (P1)", gp->tr[0]);
	break; 
      case TRI_P12:
      case TRI_Ave:
      case TRI_Acc:
      case TRI_Dif:
      case TRI_LP1:
      case TRI_AvgN:
      case TRI_AvgN1:
      case TRI_AccN1:
      case TRI_AvgN2:
      case TRI_AvgN3:
      case TRI_AccN3:
	printf("   %24s : %d\n", "Time 1 (P1)", gp->tr[0]);
	printf("   %24s : %d\n", "Time 2 (P2)", gp->tr[1]);
	break; 
    }
    printf("   %24s : %d\n", "Decimal Scale Factor", gp->scale10);
    printf("   %24s : %d\n", "Binary Scale Factor", gp->bd->bscale);
    printf("   %24s : %f\n", "Reference Value", gp->bd->ref);
*/
 /* all done in float to reproduce FSL value */
 /*   {				
	float dscal;
	dscal = gp->scale10;
	dscal = pow(10.0, dscal);
	sprintf(format, "   %%24s : %%.%dg\n", prec);
	printf(format, "Minimum Value", (float)(gp->bd->ref / dscal));
    }
    printf("   %24s : %d\n", "Number of Bits", gp->bd->nbits);
    printf("   %24s : %s\n", "BMS Included", gp->has_bms ? "TRUE" : "FALSE");
*/
 
/*
    printf("   %24s : %s\n", "GDS Included", gp->has_gds ? "TRUE" : "FALSE");
*/

    l_gb->nhas_gds= AddInt( l_gb->has_gds, l_gb->nhas_gds, gp->has_gds );
    l_gb->nparam_vers= AddInt( l_gb->param_vers, l_gb->nparam_vers,gp->param_vers);
} 



static int AddInt( int* arr, int narr, int val ){
/* This is an utility function local to this file used by the function 
   add_grib. */ 
       
    int i=0;
    int b=0;

    if( !(narr%NR) && narr !=0 ){	        
	if(!(arr=(int*)realloc(arr,(sizeof(int)*(narr + NR)))))
        uerror("Error: can't alloc for creftime array: %d", narr);

	for( i=narr; i<(narr + NR); i++ )
		arr[i]=0;
    }

    
    for( i=0; i< narr; i++ ){
	if( val == arr[i] ){
	   b=1; break;
	}
    }

    if(!b){
	arr[narr]= val;
	narr++;
    }

  return narr;
} 


static int AddCharp( char** arr, int narr, char* val ){
/* This is an utility function local to this file used by the function 
   add_grib. */ 
       
    int i=0;
    int b=0;

    if( !(narr%NR) && narr !=0 ){	        
	if(!(arr=(char**)realloc(arr,(sizeof(char*)*(narr + NR)))))
        uerror("Error: can't alloc for creftime array: %d", narr);

	for( i=narr; i<(narr + NR); i++ )
		arr[i]=NULL;
    }

    for( i=0; i< narr; i++ ){
	if(!strcmp( arr[i], val )){
	   b=1; break;
	}
    }

    if(!b){
	arr[narr]= val;
	narr++;
    }

  return narr; 
}



    

/*
 * Parse raw product bytes into product_data structure.  Returns 0 if
 * failed.  User should call free_product_data() on result when done with
 * it.  Also expands quasi-regular grids to full rectangular grids if qmeth
 * is non-null.
 */
static product_data *
grib_decode(prodp, quasp)
     prod *prodp ;		/* input raw product bytes */
     quas *quasp ;		/* if non-null, method used to expand
				   quasi-regular "grids" */
{
    grib1 *gp = new_grib1(prodp); /* overlay raw bits on raw grib1 structure */
    product_data *pdp;

    if (gp == 0)
	return 0;

    pdp = new_product_data(gp);	/* compute cooked product structure, with GDS
				   (manufactured, if necessary) and bytemap */
    free_grib1(gp);

    if (!pdp)
	return 0;

    if (pdp->gd->quasi && quasp) {
	int ret = expand_quasi(quasp, pdp) ; /* Changes *pdp */
	if (!ret) {
	    uerror("%s: can't expand quasi-regular grid", pdp->header);
	    return 0;
	}
    }
    return pdp;
}


static int
do_add (fp, ep, timeout, prec, quasp)
/* This function is closely based on the fubction do_dump in the file gribdump.c
   ang gets, decodes and stores the header information from the gribobject */
    FILE *fp;			/* input */
    FILE *ep;			/* if non-null, where to append bad GRIBs */
    int timeout;
    int prec;			/* precision to use for printing floats */
    quas *quasp;		/* if non-null, quasi-reg expansion method */
{
    struct prod the_prod;
    struct product_data *gribp;	/* UPC structure */
    
    num_wmo_messages = 0;
    num_gribs_decoded = 0;

    if (brief_output)
	printf(" cnt mdl grd prm    lvlf lev1 lev2  trf tr0 tr1 bits bms gds   npts header\n");

    TEST_NOSERC("tralla",1000);

    while(1){
        int bytes= add_get_prod(fp, timeout, &the_prod);

        if( bytes == -1)
	    return -1;

	if (bytes == 0){
	  TEST_NOSERC( " bytes == 0", num_gribs_written );
	  continue;/*continue;*/
        }
	else
	  num_wmo_messages++;
	
	/* decode it into a grib1 product structure */
	gribp = grib_decode(&the_prod, quasp);
	if (gribp == 0) {
	    TEST_NOSERC( "num_gribs_written gribp == 0", num_gribs_written );
	    if (ep) {
		if (fwrite(the_prod.bytes, the_prod.len, 1, ep) == 0) {
		    serror("writing bad GRIB to error file");
		}
	    }
	    continue;/*continue;*/
	}
	num_gribs_decoded++;

	/* Dump grib info to stdout */
	/*if(brief_output)
	    print_grib_line(gribp);
	else
	*/
	
	if( gdes_analyse )
	    add_grib(gribp,prec, 1);
	else
	    add_grib(gribp, prec, 0);
	free_product_data(gribp);
	num_gribs_written++;

	TEST_NOSERC("num_gribs_written", num_gribs_written );
    }
}


int
main(ac,av)
     int ac ;
     char *av[] ;
{
    /* This main function is similar to the main in the gribdump.c */
    int counter=0;
    extern int optind;
    char *logfname = "-" ;	/* log file name, default is stdout */
    FILE *fp = stdin;		/* default input */
    FILE *ep = 0;		/* file handle for bad GRIBS output, when
				   -e badfname used */
    int prec = DEFAULT_PRECISION; /* precision to use for printing floats */
    int timeo = DEFAULT_TIMEOUT ; /* timeout */
    quas *quasp = 0;		/* default, don't expand quasi-regular grids */
    /*
     * register exit handler
     */
    init_gribdata(&m_gb, NR_GDES );
    m_lgb.ngb=0;
    m_lgb.gb=(gribdata*)malloc(sizeof(gribdata)*NR_GDES);/* allocate memory for the list of gribdata*/
   
    /*TEST_NOSERC("HVA SKJER?",0 );*/

    if(atexit(cleanup) != 0)
      {
	  serror("atexit") ;
	  exit(1) ;
      }
    
    /*
     * set up signal handlers
     */
    set_sigactions() ;
    
    {
	extern int opterr;
	extern char *optarg;
	int ch;
	int logmask = (LOG_MASK(LOG_ERR) | LOG_MASK(LOG_NOTICE)) ;
	int errflg = 0;
	
	opterr = 1;
	
	brief_output = 0;
        headers_only = 0;
	testNoserc=0;
	gdes_analyse=0;
        /* Default is to generate CDL */
        printcdl=1;
        
        // New option "-g gribtonctable" implemented May 2002 by Arild Burud
	//while ((ch = getopt(ac, av, "vxhwyzbl:t:e:p:q:")) != EOF) {
	while ((ch = getopt(ac, av, "vxhwyzbl:t:e:p:q:g:")) != EOF) {
	    switch (ch) {
	    case 'v': /* verbose */
		logmask |= LOG_MASK(LOG_INFO) ;
		break;
	    case 'x': /* ? */
		logmask |= LOG_MASK(LOG_DEBUG) ;
		break;
	    case 'g': /* gribtonctable */
	      /* fprintf(stderr, "Conversion table in file %s\n",optarg) ; */
                user_makeparamtable(optarg);
                /* user_printparamtable(); */
		break;
	    case 'l': /* logfile */
		logfname = optarg ;
		break;
	    case 't': /* timeout */
		timeo = atoi(optarg) ;
		if(timeo < 1) {
		    fprintf(stderr, "%s: invalid timeout %s",
			    av[0], optarg) ;
		    errflg++;
		}
		break;
	    case 'h': /* headers */
		headers_only = 1;
		break;
	    case 'b': /* brief */
		brief_output = 1;
		headers_only = 1;
		break;
	    case 'e': /* errorfile */
		ep = fopen(optarg, "w");
		if(!ep) {
		    serror("can't open %s", optarg);
		    errflg++;
		}
		break;
	    case 'p': /* precision */
		prec = atoi(optarg) ;
		if(prec < 1 || prec > 99) {
		    fprintf(stderr, "% s: invalid precision %s",
			    av[0], optarg) ;
		    errflg++;
		}
		break;
	    case 'q': /* grid expansion */
		quasp = qmeth_parse(optarg);
		if(!quasp) {
		    fprintf(stderr,
			    "%s: invalid quasi-regular expansion method %s",
			    av[0], optarg) ;
		    errflg++;
		}
		break;
	    case 'z': /* debug */
		testNoserc=1;
		break;
	    case 'y': /* debug */
		gdes_analyse=1;
		break;
	    case 'w': /* produce CDL */
		printcdl=1;
		break; 
	    case '?': /* print help */
		errflg++;
		break;
	    }
	}

	if(errflg)
	  usage(av[0]);	
	(void) setulogmask(logmask) ;
    }
        
    /*
     * initialize logger
     */
    (void) openulog(ubasename(av[0]),
		    (LOG_CONS|LOG_PID), LOG_LOCAL0, logfname) ;
    uinfo("Starting Up") ;

    do {
	if (optind < ac) {	/* arguments still left, use as filenames */
	    fp = fopen(av[optind++], "r");
	    if(!fp) {
		serror("can't open %s", av[optind-1]);
		return 1;
	    }
            grib_file_name=av[optind-1]; /* added 18.12.2001 by a.b. */
	}
	do_add(fp, ep, timeo, prec, quasp);

	if(testNoserc){
	    counter++;
	    printf("   %24s : %d\n","counter", counter ); 
	}  
    } while (optind < ac);

    if( gdes_analyse )
	print_gribdata(1);
    else if(printcdl)
	print_cdl();
    else
	print_gribdata(0);

    
    return 0;
}

/*
static void test_realloc( int reftime){
    if( !(reftime%NR) ){
	int i;
        char** test=NULL;
        int nr=NR;
	size_t mysize;
	char** newpointer;
	int tt=(int)(reftime/nr);
	printf("NR=%d  tt=%d \n", nr, tt);
        
        printf("total nr=%d \n", nr*(tt+1) );
        mysize = (size_t)(sizeof(char*)*nr*(tt+1));

	test=(char**)malloc(sizeof(char*)*NR);
	for( i=0; i < 250; i++ )
	    test[i]="ppp";

	printf("test[i]=\"ppp\";\n");

	for( i=0; i < 250; i++ )
	    printf("%s", test[i]);
	printf("printf\n");
	newpointer=(char**)realloc(test,mysize);
	  
	    if(!(newpointer))
		printf("ERROR: realloc nreftime; NR=%d tt=%d \n", nr, tt);

	    for( i=nr*tt; i<nr*(tt+1); i++ )
		newpointer[i]=NULL;

	    
	    printf("newpointer=%d", newpointer );
	    printf("test=%d", test );
	    
	    if( tt==0 ){
		for( i=0; i<nr; i++ )
		    newpointer[i]="ttt";
	    }
		
	    else
	    for( i=nr; i<nr*(tt+1); i++ )
		newpointer[i]="ttt";


	    

	    for( i=0; i < nr*(tt+1); i++ )
		printf("%s", newpointer[i]);

	    printf("printf5\n");

	free(test);
	test=NULL;
	free(newpointer);
	newpointer=NULL;
}
}
*/   
